library jaguar_auth.session;

import 'dart:io';
import 'dart:async';
import 'package:jaguar/jaguar.dart';
import 'package:jaguar_jwt/jaguar_jwt.dart';

part 'common.dart';

/// JWT based session manager.
///
/// Use [io] to configure how session data is transported. Built-in options are:
/// 1. [SessionIoCookie]: Stores token in cookie
/// 2. [SessionIoAuthHeader]: Stores token in authorization header
/// 3. [SessionIoHeader]: Stores token in header
///
/// By default, [JwtSession] uses [SessionIoAuthHeader].
///
///     server() async {
///       final server = Jaguar(sessionManager: JwtSession(jwtConfig));
///       server.add(reflect(LibraryApi()));
///       await server.serve();
///     }
class JwtSession extends Object with _BaseJwtSession implements SessionManager {
  final SessionIo io;

  /// JWT configuration used to issue a JWT token
  final JwtConfig config;

  /// Information required to validate a JWT token
  final JwtValidationConfig validationConfig;

  JwtSession(this.config,
      {this.validationConfig: const JwtValidationConfig(),
      this.io: const SessionIoAuthHeader()});

  /// Parses session from the given [request]
  Future<Session> parse(Context context) async {
    String raw = io.read(context);
    if (raw == null) return Session.newSession({});
    return decodeJwt(raw);
  }

  /// Writes [response] with session details
  void write(Context context) {
    if (!context.sessionNeedsUpdate) return;

    final Session session = context.parsedSession;
    io.write(context, encodeJwt(session.asMap));
  }
}
